# Asnpf help to hit some non spoofed email for domains that doesnt have SPF or DKIM configured and doesnt hit mxpf
# It compare the sender ip asn with the domain mx record ip asn

package Mail::SpamAssassin::Plugin::asnpf;
1;

use strict;
use Socket;
use Net::DNS;
use Mail::SpamAssassin;
use Mail::SpamAssassin::Logger;
use Mail::SpamAssassin::Plugin;
use Mail::SpamAssassin::Message::Metadata::Received qw(parse_received_line);
use Mail::SpamAssassin::Util qw(reverse_ip_address);
our @ISA = qw(Mail::SpamAssassin::Plugin);


sub new {
 my ($class, $mailsa) = @_;
 $class = ref($class) || $class;
 my $self = $class->SUPER::new( $mailsa );
 bless ($self, $class);
 $self->register_eval_rule ( 'check_asnpf' );
 return $self;
}

sub check_asnpf {
 my ($self, $msg) = @_;
 my $msgr = $msg->get_message();
 # Get untrusted relay ip
 my @untrusted = @{$msgr->{metadata}->{relays_untrusted}};
 my $relay = shift(@untrusted);
 my $msgip = $relay->{ip};

 # Go on only if msgip have a value
 if (defined($msgip)) {
  dbg( "asnpf: Message Ip : " . $msgip );
  # Get cymru asn of message ip
  my $msgipr = reverse_ip_address($msgip) . ".origin.asn.cymru.com" ;
  #dbg#dbg( "asnpf: Message Ip Reversed : " . $msgipr );
  my $res = Net::DNS::Resolver->new;
  my $reply = $res->search( $msgipr, "TXT");
  my $msgasn = 0;
  foreach my $record ($reply->answer) {
   $msgasn = substr($record->txtdata, 0, index($record->txtdata, ' |'));
  }
  dbg( "asnpf: MsgAsn : " . $msgasn );

  # Get from address and split into user and domain 
  my $msgfrom = $msg->get( 'From:addr' );
  # Go on only if msgfrom contains at least a @ and a .
  if (index ($msgfrom, "@" ) != -1 && index ($msgfrom, "." ) != -1) {
   #dbg# Mail::SpamAssassin::Plugin::dbg( "asnpf: Message From : " . $msgfrom );
   my ($msguser, $msgdomain) = $msgfrom =~ /(.*)@(.*)/;
   #dbg# dbg( "asnpf: User : " . $msguser );
   #dbg# dbg( "asnpf: Domain : " . $msgdomain );
   # Search MX record for from domain
   my $amx="";
   my $aip="";
   my $mxname="";
   my $mxip="";
   my $mxipr;
   my $resmx = Net::DNS::Resolver->new;
   my $resip = Net::DNS::Resolver->new;
   # Dns Timeouts
   $resmx->udp_timeout(3);
   $resmx->tcp_timeout(3);
   $resip->udp_timeout(3);
   $resip->tcp_timeout(3);
   # Scan mx records
   my @mx = mx($resmx, $msgdomain);
   if (@mx) {
    foreach $amx (@mx) {
     $mxname=$amx->exchange;
     #dbg#dbg("asnpf: Mx : " . $mxname);
     # Resolve ip of mx
     my $ip = $resip->search($mxname);
     if ($ip) {
      foreach $aip ($ip->answer) {
       $mxip=$aip->address;
       dbg("asnpf: Mx Ip : " . $mxip);
       
       # Get cymru asn
       my $mxipr = reverse_ip_address($mxip) . ".origin.asn.cymru.com" ;
       #dbg#dbg( "asnpf: Mxip Ip Reversed : " . $mxipr );
       my $res = Net::DNS::Resolver->new;
       my $reply = $res->search( $mxipr, "TXT");
       my $mxasn = 0;
       foreach my $record ($reply->answer) {
        $mxasn = substr($record->txtdata, 0, index($record->txtdata, ' |'));
       }
       dbg( "asnpf: MxAsn : " . $mxasn );

       # Comparing mxasn with msgasn
       if ($mxasn eq $msgasn) {
        # Return 1 and close if match
        dbg( "asnpf:  END WITH MATCH" );
        return 1;
       }
      }
     }
    }
   }
  }
 }
 dbg( "asnpf:  END WITHOUT MATCH" );

return 0;
}

